aboutsummaryrefslogtreecommitdiff
path: root/gold/plugin.cc
diff options
context:
space:
mode:
Diffstat (limited to 'gold/plugin.cc')
-rw-r--r--gold/plugin.cc77
1 files changed, 65 insertions, 12 deletions
diff --git a/gold/plugin.cc b/gold/plugin.cc
index 8f6e87f..3ccd8d0 100644
--- a/gold/plugin.cc
+++ b/gold/plugin.cc
@@ -78,6 +78,9 @@ static enum ld_plugin_status
get_symbols(const void *handle, int nsyms, struct ld_plugin_symbol *syms);
static enum ld_plugin_status
+get_symbols_v2(const void *handle, int nsyms, struct ld_plugin_symbol *syms);
+
+static enum ld_plugin_status
add_input_file(const char *pathname);
static enum ld_plugin_status
@@ -156,7 +159,7 @@ Plugin::load()
sscanf(ver, "%d.%d", &major, &minor);
// Allocate and populate a transfer vector.
- const int tv_fixed_size = 23;
+ const int tv_fixed_size = 24;
int tv_size = this->args_.size() + tv_fixed_size;
ld_plugin_tv* tv = new ld_plugin_tv[tv_size];
@@ -228,6 +231,10 @@ Plugin::load()
tv[i].tv_u.tv_get_symbols = get_symbols;
++i;
+ tv[i].tv_tag = LDPT_GET_SYMBOLS_V2;
+ tv[i].tv_u.tv_get_symbols = get_symbols_v2;
+
+ ++i;
tv[i].tv_tag = LDPT_ADD_INPUT_FILE;
tv[i].tv_u.tv_add_input_file = add_input_file;
@@ -810,11 +817,11 @@ Pluginobj::Pluginobj(const std::string& name, Input_file* input_file,
{
}
// Return TRUE if a defined symbol might be reachable from outside the
// universe of claimed objects.
static inline bool
-is_visible_from_outside(Symbol* lsym)
+is_referenced_from_outside(Symbol* lsym)
{
if (lsym->in_real_elf())
return true;
@@ -822,6 +829,15 @@ is_visible_from_outside(Symbol* lsym)
return true;
if (parameters->options().is_undefined(lsym->name()))
return true;
+ return false;
+}
+
+// Return TRUE if a defined symbol might be reachable from outside the
+// load module.
+
+static inline bool
+is_visible_from_outside(Symbol* lsym)
+{
if (parameters->options().export_dynamic() || parameters->options().shared())
return lsym->is_externally_visible();
return false;
@@ -830,8 +846,18 @@ is_visible_from_outside(Symbol* lsym)
// Get symbol resolution info.
ld_plugin_status
-Pluginobj::get_symbol_resolution_info(int nsyms, ld_plugin_symbol* syms) const
-{
+Pluginobj::get_symbol_resolution_info(int nsyms,
+ ld_plugin_symbol* syms,
+ int version) const
+{
+ // For version 1 of this interface, we cannot use
+ // LDPR_PREVAILING_DEF_IRONLY_EXP, so we return LDPR_PREVAILING_DEF
+ // instead.
+ const ld_plugin_symbol_resolution ldpr_prevailing_def_ironly_exp
+ = (version > 1
+ ? LDPR_PREVAILING_DEF_IRONLY_EXP
+ : LDPR_PREVAILING_DEF);
+
if (nsyms > this->nsyms_)
return LDPS_NO_SYMS;
@@ -862,9 +888,14 @@ Pluginobj::get_symbol_resolution_info(int nsyms, ld_plugin_symbol* syms) const
if (lsym->source() != Symbol::FROM_OBJECT)
res = LDPR_RESOLVED_EXEC;
else if (lsym->object()->pluginobj() == this)
- res = (is_visible_from_outside(lsym)
- ? LDPR_PREVAILING_DEF
- : LDPR_PREVAILING_DEF_IRONLY);
+ {
+ if (is_referenced_from_outside(lsym))
+ res = LDPR_PREVAILING_DEF;
+ else if (is_visible_from_outside(lsym))
+ res = ldpr_prevailing_def_ironly_exp;
+ else
+ res = LDPR_PREVAILING_DEF_IRONLY;
+ }
else if (lsym->object()->pluginobj() != NULL)
res = LDPR_RESOLVED_IR;
else if (lsym->object()->is_dynamic())
@@ -878,9 +909,14 @@ Pluginobj::get_symbol_resolution_info(int nsyms, ld_plugin_symbol* syms) const
if (lsym->source() != Symbol::FROM_OBJECT)
res = LDPR_PREEMPTED_REG;
else if (lsym->object() == static_cast<const Object*>(this))
- res = (is_visible_from_outside(lsym)
- ? LDPR_PREVAILING_DEF
- : LDPR_PREVAILING_DEF_IRONLY);
+ {
+ if (is_referenced_from_outside(lsym))
+ res = LDPR_PREVAILING_DEF;
+ else if (is_visible_from_outside(lsym))
+ res = ldpr_prevailing_def_ironly_exp;
+ else
+ res = LDPR_PREVAILING_DEF_IRONLY;
+ }
else
res = (lsym->object()->pluginobj() != NULL
? LDPR_PREEMPTED_IR
@@ -1411,7 +1447,24 @@ get_symbols(const void* handle, int nsyms, ld_plugin_symbol* syms)
Pluginobj* plugin_obj = obj->pluginobj();
if (plugin_obj == NULL)
return LDPS_ERR;
- return plugin_obj->get_symbol_resolution_info(nsyms, syms);
+ return plugin_obj->get_symbol_resolution_info(nsyms, syms, 1);
+}
+
+// Version 2 of the above. The only difference is that this version
+// is allowed to return the resolution code LDPR_PREVAILING_DEF_IRONLY_EXP.
+
+static enum ld_plugin_status
+get_symbols_v2(const void* handle, int nsyms, ld_plugin_symbol* syms)
+{
+ gold_assert(parameters->options().has_plugins());
+ Object* obj = parameters->options().plugins()->object(
+ static_cast<unsigned int>(reinterpret_cast<intptr_t>(handle)));
+ if (obj == NULL)
+ return LDPS_ERR;
+ Pluginobj* plugin_obj = obj->pluginobj();
+ if (plugin_obj == NULL)
+ return LDPS_ERR;
+ return plugin_obj->get_symbol_resolution_info(nsyms, syms, 2);
}
// Add a new (real) input file generated by a plugin.
@@ -810,11 +817,11 @@ Pluginobj::Pluginobj(const std::string& name, Input_file* input_file,
{
}
-// Return TRUE if a defined symbol might be reachable from outside the
+// Return TRUE if a defined symbol is referenced from outside the
// universe of claimed objects.
static inline bool